from datetime import date, datetime
from typing import Optional
from urllib import parse

from .utils import calculate_hash, concatenate_params, get_amount_string, get_timestamp


class AuthorizeClient:
    """A client for interacting with the ITS PCI Gateway for Authorize.net."""

    def __init__(self, endpoint: str, order_type: str, key: str) -> None:
        """Initializes an AuthorizeClient instance.

        Args:
            endpoint: The location of the PCI Gateway API, including trailing slash.
            order_type: The order type assigned by ITS.
            key: The API key assigned by ITS.
        """
        self.endpoint = endpoint
        self.order_type = order_type
        self.key = key

    def get_payment_url(
        self,
        *,
        ref_id: str,
        amount: int,
        redirect_url: str,
        cancel_url: str,
        timestamp: Optional[datetime] = None,
    ) -> str:
        """Generate a PCI Gateway URL for initiating a one-time payment."""
        query_data = {
            "order_type": [self.order_type],
            "refId": [ref_id],
            "amount": [get_amount_string(amount)],
            "redirect_url": [redirect_url],
            "cancel_url": [cancel_url],
            "timestamp": [str(get_timestamp(timestamp))],
        }
        query_data["order_hash"] = [
            calculate_hash(concatenate_params(query_data), self.key)
        ]
        query_string = parse.urlencode(query_data, doseq=True)
        return parse.urljoin(
            f"{self.endpoint}authApi/authorize/payment/", f"?{query_string}"
        )

    def get_subscription_url(
        self,
        *,
        ref_id: str,
        length: int,
        unit: str,
        total_occurances: int,
        amount: int,
        redirect_url: str,
        merchant_customer_id: str,
        start_date: Optional[date] = None,
        invoice_number: Optional[str] = None,
        name: Optional[str] = None,
        trial_occurances: Optional[int] = None,
        trial_amount: Optional[int] = None,
        customer_address_id: Optional[int] = None,
        timestamp: Optional[datetime] = None,
    ) -> str:
        """Generate a PCI Gateway URL for initiating a subscription.

        Args:
            ref_id: A unique ID for the transaction. Maximum length of 25 characters.
            length: The length of the subscription, between 0 and 999.
            unit: The unit of time for the subscription length (e.g., days, months).
            total_occurances: The total number of occurrences for the subscription,
                between 0 and 9999.
            amount: The price in cents for the subscription. Maximum of 15 digits.
            redirect_url: The URL to which the user will be redirected after processing.
            merchant_customer_id: A merchant customer ID, with a maximum length of 20
                characters. This should be unique for every new subscription order.
            start_date: The start date of the subscription. Defaults to today.
            invoice_number: Invoice number. Maximum length of 20 characters.
            name: An optional name, with a maximum length of 25 characters.
            trial_occurances: The number of trial occurrences, between 0 and 99.
            trial_amount: The trial price in cents. Maximum of 15 digits.
            customer_address_id: An optional customer address ID.
            timestamp: The timestamp when the request is created. Defaults to now.
        """
        if start_date is None:
            start_date = date.today()
        query_data = {
            "order_type": [self.order_type],
            "refId": [ref_id],
            "length": [str(length)],
            "unit": [unit],
            "start_date": [start_date.strftime("%Y-%m-%d")],
            "total_occurances": [str(total_occurances)],
            "amount": [get_amount_string(amount)],
            "redirect_url": [redirect_url],
            "merchant_customerId": [merchant_customer_id],
            "timestamp": [str(get_timestamp(timestamp))],
        }
        if invoice_number is not None:
            query_data["invoice_number"] = [invoice_number]
        if name is not None:
            query_data["name"] = [name]
        if trial_occurances is not None:
            query_data["trial_occurances"] = [str(trial_occurances)]
        if trial_amount is not None:
            query_data["trial_amount"] = [get_amount_string(trial_amount)]
        if customer_address_id is not None:
            query_data["customer_addressId"] = [str(customer_address_id)]
        query_data["order_hash"] = [
            calculate_hash(concatenate_params(query_data), self.key)
        ]
        query_string = parse.urlencode(query_data, doseq=True)
        return parse.urljoin(
            f"{self.endpoint}authApi/subscription/create/", f"?{query_string}"
        )
