from collections.abc import Sequence
from datetime import datetime
from hashlib import sha256
from typing import Optional

from .constants import PARAM_ORDER
from .types import QueryData


def get_amount_string(total_in_cents: int) -> str:
    """Converts a total (in cents), to a string appropriate for PCI Gateway API calls.

    The ITS PCI Gateway only accepts amounts with a maximum of 15 digits, including two
    decimal places.
    """
    if total_in_cents < 0:
        raise ValueError("Total can not be negative.")
    if total_in_cents > 999999999999999:
        raise ValueError("Total exceeds maximum of 15 digits.")
    dollars, cents = divmod(total_in_cents, 100)
    return f"{dollars:01d}.{cents:02d}"


def get_timestamp(dt: Optional[datetime] = None) -> int:
    """Returns milliseconds since epoch."""
    if dt is None:
        dt = datetime.now()
    return int(round(dt.timestamp()))


def concatenate_params(params: QueryData, order: Sequence[str] = PARAM_ORDER) -> str:
    """Generates a concatenated string from query parameter values."""
    output = ""
    for param in order:
        output += "".join(params.get(param, []))
    return output


def calculate_hash(message: str, key: str) -> str:
    """Calculate a SHA256 hash given a message and a key."""
    string = message + key
    return sha256(string.encode("utf-8")).hexdigest()
